/*
 * phase.c
 *
 * Created: 30.01.2018 14:15:10
 *  Author: dsosnowski
 */ 


#define F_CPU 8000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <avr/pgmspace.h>
#include "phase.h"

//---------------------------------------------------
#define PHASE_PORT		PORTD
#define PHASE_DDR		DDRD
#define PHASE_PIN		PIND
#define PHASE_IN		2		//(INT0)
#define PHASE_OUT		5
#define PHASE_ON		(PHASE_PORT &= ~(1<<PHASE_OUT))
#define PHASE_OFF		(PHASE_PORT |= (1<<PHASE_OUT))

#define RANGE_REG		OCR1A
#define POWER_REG		OCR1B

#define POWER_COMP_RANGE			10000
#define POWER_OFFSET_CORRECTION		200

struct {
	uint16_t comp;		//half sine = 10000
} phase;

//---------------------------------------------------
void PhaseInit(void){
	PHASE_PORT |= (1<<PHASE_IN);
	PHASE_OFF;
	PHASE_DDR |= (1<<PHASE_OUT);
	
	MCUCR = (1<<ISC01 | 0<<ISC00);
	GIMSK = (1<<INT0);
	
	RANGE_REG = 11000;
	POWER_REG = 0xFFFF;
	TCCR1A = (0<<WGM11 | 0<<WGM10);
	TCCR1B = (0<<WGM13 | 1<<WGM12 | 0);
	TIMSK = (1<<OCIE1A | 1<<OCIE1B);
	sei();
}

//---------------------------------------------------
//val range 0...100 %
void PhasePowerSet(uint8_t val){
	uint16_t calc = 0;
	calc = POWER_COMP_RANGE;
	calc -= (val * 100);
	
	if (calc > POWER_OFFSET_CORRECTION){
		calc -= POWER_OFFSET_CORRECTION;
	} else {
		calc = 0;
	}

	if (calc >= (POWER_COMP_RANGE - POWER_OFFSET_CORRECTION)){
		calc = 0xFFFF;
	}

	phase.comp = calc;
}

//---------------------------------------------------
ISR(INT0_vect){
	GTCCR = (1<<PSR10);
	TCNT1 = 0;
	if (phase.comp == 0) {
		PHASE_ON;
		POWER_REG = 0xFFFF;
	} else {
		PHASE_OFF;
		POWER_REG = phase.comp;
	}
	TCCR1B = (0<<WGM13 | 1<<WGM12 | 2);
}

//---------------------------------------------------
// triac ON
ISR(TIMER1_COMPB_vect){
	PHASE_ON;
}

//---------------------------------------------------
//overrange sine time
ISR(TIMER1_COMPA_vect){
	TCNT1 = 0;
	PHASE_OFF;
	TCCR1B = (0<<WGM13 | 1<<WGM12 | 2);
}

//---------------------------------------------------
void PhaseSet(){
	
}